#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>
#include "lcd.h"


#define F_CPU 16000000
#define nivel_max = 5;


int led[6] = {PA0, PA1, PA2, PA3, PA4, PA5 };

int seq[5][5];
int nivel = 1;
int reset = 0;
int nr = 0;


void port_init()
{
	// setez pinii PB ca intrare
	DDRB = 0;
	DDRB &= ~(1 << PB0);
	DDRB &= ~(1 << PB1);
	DDRB &= ~(1 << PB2);
	DDRB &= ~(1 << PB3);
	DDRB &= ~(1 << PB4);
	DDRB &= ~(1 << PB5);
	DDRB &= ~(1 << PB6);
	
	// activez rezistenta de pull-up pentru pinii PB
	PORTB = 0;
	PORTB |= (1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB4) | (1 << PB5) | (1 << PB6);

	// setez pinii PA ca iesire
	DDRA = 255;
	
	// opresc LED-ul pe de pinii PA
	PORTA = 0;
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
	PORTA &= ~(1 << PA2);
	PORTA &= ~(1 << PA3);
	PORTA &= ~(1 << PA4);
	PORTA &= ~(1 << PA5);
	PORTA &= ~(1 << PA6);
	PORTA &= ~(1 << PA7); 
	
	PINB = 0;
}


void generate_seq() {
	int i,j;
	srand(nr);
	nr++;
	if (nr == 1000)
			nr = 1;
	for (i = 0; i < 6; i++) {
		for (j = 0; j < 6; j++)
			seq[i][j] = rand() % 6;
	}
}



int find_button() {
	int apasat = 0;
	while (apasat == 0) {
		if ((PINB & (1 << PB0)) == 0) {
			while ((PINB & (1 << PB0)) == 0);
			apasat = 1;
			return 0;
		}
		else if ((PINB & (1 << PB1)) == 0) {
			while ((PINB & (1 << PB1)) == 0);
			apasat = 1;
			return 1;
		}
		else if ((PINB & (1 << PB2)) == 0) {
			while ((PINB & (1 << PB2)) == 0);
			apasat = 1;
			return 2;
		}
		if ((PINB & (1 << PB3)) == 0) {
			while ((PINB & (1 << PB3)) == 0);
			apasat = 1;
			return 3;
		}
		else if ((PINB & (1 << PB4)) == 0) {
			while ((PINB & (1 << PB4)) == 0);
			apasat = 1;
			return 4;
		}
		
		else if ((PINB & (1 << PB5)) == 0) {
			while ((PINB & (1 << PB5)) == 0);
			apasat = 1;
			return 5;
		}
		else if ((PINB & (1 << PB6)) == 0) {
			while ((PINB & (1 << PB6)) == 0);
			apasat = 1;
			return 6;
		}
		
	}
	return -1;
}



int main() {
	
	port_init();
	LCD_init();
	generate_seq();
	LCD_print("Start!");
	while(find_button() != 6);
	
	
	while (1) {
		
	
		char buf[1];	
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		LCD_print("Level:");
		LCD_printAt(6, itoa(nivel,buf,10));
		
		int i;
		// aprinde secventa de leduri in functie de nivelul curent
		for (i = 0; i < nivel + 1; i++ ) {
			PORTA |= (1<<led[seq[nivel-1][i]]);
			_delay_ms(400);
			PORTA &= ~(1<<led[seq[nivel-1][i]]);
			_delay_ms(200);
		}
		
		// asteapta refacerea secventei
		int count = 0;
		int ok = 1;
		
		
		// cat timp se apasa butonul corect si nu s-a depasit nr de butoane care trebuie apasate
		while (count < nivel+1 && ok == 1) {
			
			int apasat = find_button();
			// daca se apasa RESET
			if (apasat == 6)  {
				generate_seq();
				nivel = 1;
				ok = -1;
				break;
			}
			
			// aprind led buton apasat
			PORTA |= (1<<led[apasat]);
			_delay_ms(200);
			// il sting
			PORTA &= ~(1<<led[apasat]);
			_delay_ms(200);
			
			if (seq[nivel-1][count] != apasat) {
				ok = 0;
				break;
			} else 
				count++;
		}
		
		// ultimul nivel terminat - joc castigat, pentru a relua jocul se apasa RESET
		if (nivel == 5 && ok == 1) {
			generate_seq();
			nivel = 1;
			LCD_printAt(0x40, "Congrats!!!");
			//int apasat = find_button();
			//while (apasat != 6);
			while(find_button() != 6);
			_delay_ms(300);
			
		}
		
		else if (ok == 1 && nivel < 5) {
			PORTA |= (1<<PA6);
			_delay_ms(200);
			PORTA &= ~(1<<PA6);
			_delay_ms(200);
			PORTA |= (1<<PA6);
			_delay_ms(200);
			PORTA &= ~(1<<PA6);
			nivel++;
			//afisez pe ecran mesaj pentru a se trece la nivelul urmator
			LCD_printAt(0x40, "Next level!");
			while(find_button() != 6);
			_delay_ms(300);
		}
		
		
		// se ramane la nivelul curent 
		else if (ok == 0) {
			LCD_printAt(0x40, "Try again!!!");
			PORTA |= (1<<PA7);
			_delay_ms(200);
			PORTA &= ~(1<<PA7);
			_delay_ms(200);
			PORTA |= (1<<PA7);
			_delay_ms(200);
			PORTA &= ~(1<<PA7);
			while(find_button() != 6);
			_delay_ms(300);
			
		}
		else
			_delay_ms(300);
	}
	return 0;
}







